<?php

namespace App\domain\amadeus\v2;

use App\domain\amadeus\QueryFareCheckRulesAction;
use App\domain\amadeus\QueryFareInformativePriceAction;
use App\domain\amadeus\QuerySecuritySignOut;
use Exception;

class OfferPricingAction
{
    protected QueryFareInformativePriceAction $queryFareInformativePriceAction;
    protected QueryFareCheckRulesAction $queryFareCheckRulesAction;
    protected QuerySecuritySignOut $querySecuritySignOutAction;
    protected array $searchCriteriaFlight;
    protected array $passenger;
    protected array $segments;
    protected array $recommendation;

    public function __construct()
    {
        $this->queryFareInformativePriceAction = new QueryFareInformativePriceAction;
        $this->queryFareCheckRulesAction = new QueryFareCheckRulesAction;
        $this->querySecuritySignOutAction = new QuerySecuritySignOut;
    }
    public function execute(array $searchCriteria, array $offerDetails): array
    {
        $this->passenger = $searchCriteria['passenger'];
        $this->searchCriteriaFlight = $searchCriteria['searchCriteriaFlight'];
        $this->segments = $offerDetails['segments'];
        $this->recommendation = $offerDetails['recommendation'];

        $fareInformativeReply = $this->queryFareInformative();
        // Below session value generated as a side effect of queryFareInformativePriceAction 
        // and need in SellFromRecommendation and PaymentFlow actions
        $fareInformativeCombinedSegments = session('fare-informative-combined-segments');
        $otherCompany = session('other-company');

        $fareCheckRulesResponse = $this->fareCheckRules();
        $querySecuritySignOutResponse = $this->querySecuritySignOut();

        return [
            "amadeusReply" => $querySecuritySignOutResponse,
            'fareInformativeCombinedSegments' => json_decode($fareInformativeCombinedSegments),
            'otherCompany' => $otherCompany
        ];
    }

    protected function queryFareInformative(): array
    {
        return $this->queryFareInformativePriceAction->execute(
            passenger: $this->passenger,
            searchCriteriaFlight: $this->searchCriteriaFlight,
            segments: $this->segments,
            recommendation: $this->recommendation,
        );
    }

    protected function fareCheckRules(): array
    {
        return $this->queryFareCheckRulesAction->execute(
            amadeusSessionId: session("amadeus-session-id"),
            amadeusSecurityToken: session("amadeus-security-token")
        );
    }

    protected function querySecuritySignOut(): array
    {
        return $this->querySecuritySignOutAction->execute(
            amadeusSessionId: session("amadeus-session-id"),
            amadeusSecurityToken: session("amadeus-security-token")
        );
    }
}
